VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CFitnessList"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Type FitnessValues
    tileIndex As Long
    Fitness As Long
End Type

Private listSize As Long
Private item() As FitnessValues


' Loads a fitness list from a section of a file into memory
Public Sub LoadData(lngFileNum As Long)
    ' Read in the number of list entries
    Get lngFileNum, , listSize

    ' Check if there are any list items to read in
    If listSize > 0 Then
        ' Allocate array space for all list items
        ReDim item(listSize - 1) As FitnessValues

        ' Read in the fitness values
        Get lngFileNum, , item
    End If
End Sub

' Saves a fitness list to a section of a file
Public Sub SaveData(lngFileNum As Long)
    ' Write out the number of list entries
    Put lngFileNum, , listSize

    ' Check if there are any list items to write out
    If listSize > 0 Then
        ' Write out the fitness values
        Put lngFileNum, , item
    End If
End Sub

' Returns the size of this fitness list
Public Property Get Size() As Long
    Size = listSize
End Property

' Returns the tile at a given list index
' Note: Returns 0 if the index is out of range
Public Property Get tileIndex(index As Long) As Long
    ' Bounds check the access
    If index < 0 Or index >= listSize Then
        ' Bounds error, return 0
        tileIndex = 0
        Exit Property
    End If
    
    tileIndex = item(index).tileIndex
End Property

' Returns the fitness of a tile at the given list position
' Note: Returns 0 if the index is out of range
Public Property Get Fitness(index As Long) As Long
    ' Bounds check the access
    If index < 0 Or index >= listSize Then
        ' Bounds error, return 0
        Fitness = 0
        Exit Property
    End If
    
    Fitness = item(index).Fitness
End Property

' Returns the fitness value of a given tile in this list
' Note: Returns 0 if the item is not found
Public Property Get FitnessOfTile(tileIndex As Long) As Long
    Dim index As Long

    ' Search the list for the tile index
    index = FindIndexOfTile(tileIndex)
    If index <> -1 Then
        ' Return the fitness value
        FitnessOfTile = item(index).Fitness
        Exit Property
    End If

    ' Item not found. Return default value of 0
    FitnessOfTile = 0
End Property

' Adds a new item to the list
' Note: If the item already exists, the fitness is
' added to the existing item
Public Sub AddItem(tileIndex As Long, Fitness As Long)
    Dim index As Long
    
    ' Find the tile in the list
    index = FindIndexOfTile(tileIndex)
    ' Check if the tile was found
    If index <> -1 Then
        ' Add the fitness to it and exit
        item(index).Fitness = item(index).Fitness + Fitness
        Exit Sub
    End If

    ' The item was not found. Extend the list and add the item
    ReDim Preserve item(listSize) As FitnessValues
    ' Store new data
    item(listSize).tileIndex = tileIndex
    item(listSize).Fitness = Fitness
    ' Update the list size count
    listSize = listSize + 1
End Sub

' Searches the list for this tile and removes it if found
Public Sub RemoveTile(tileIndex As Long)
    Dim index As Long

    ' Find the index of the tile
    index = FindIndexOfTile(tileIndex)
    
    ' Check if the tile was found in the list
    If index <> -1 Then
        ' Remove this item from the list
        RemoveItem index
    End If
End Sub

Public Sub RemoveItem(index As Long)
    Dim i As Long
    
    ' Compact the list
    For i = index To listSize - 2
        ' Move the items back one
        item(i) = item(i + 1)
    Next i
    
    ' Shrink the size of the list
    ReDim Preserve item(listSize - 2) As FitnessValues
    ' Update the list size count
    listSize = listSize - 1
End Sub

' Finds the index into the item list of the given tile
' Note: Returns -1 if the tile was not found in the list
Public Function FindIndexOfTile(tileIndex As Long) As Long
    Dim i As Long
    
    ' Search the list for this item
    For i = 0 To listSize - 1
        ' Check if this item is the desired one
        If item(i).tileIndex = tileIndex Then
            ' Tile found. Return the index
            FindIndexOfTile = i
            Exit Function
        End If
    Next i
    
    ' Tile not found. Return -1
    FindIndexOfTile = -1
End Function
