VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CTileSelector"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public Enum tsDirection
    tsTop = 0
    tsLeft = 1
    tsRight = 2
    tsBottom = 3
End Enum

Private numTiles As Long
Private list() As CFitnessList


' Load the fitness data from file into memory
Public Sub LoadFitnessData(strFileName As String)
    Dim lngFileNum As Long
    Dim i As Long, j As Long

    ' Open the file
    lngFileNum = FreeFile()
    Open strFileName For Binary As lngFileNum

    ' Read in how much tile data it contains
    Get lngFileNum, , numTiles

    ' Allocate space to hold all the lists
    ReDim list(3, numTiles - 1) As CFitnessList

    ' Read in the lists
    For i = 0 To numTiles - 1
        For j = 0 To 3
            ' Create a new CFitnessList object
            Set list(j, i) = New CFitnessList
            ' Load the fitness data from the file
            list(j, i).LoadData lngFileNum
        Next j
    Next i

    ' Close the file
    Close lngFileNum
End Sub

' Save the fitness data to a file
Public Sub SaveFitnessData(strFileName As String)
    Dim lngFileNum As Long
    Dim i As Long, j As Long

    ' Open the file
    lngFileNum = FreeFile()
    Open strFileName For Binary As lngFileNum

    ' Write out how much tile data is present
    Put lngFileNum, , numTiles

    ' Write out the lists
    For i = 0 To numTiles - 1
        For j = 0 To 3
            ' Save the fitness data to the file
            list(j, i).SaveData lngFileNum
        Next j
    Next i

    ' Close the file
    Close lngFileNum
End Sub

' Find a suitable tile based on surrounding tiles
' Note: tile indexes of 0 are ignored
Public Function ChooseTile(topTile As Long, leftTile As Long, rightTile As Long, bottomTile As Long) As Long
    Dim list As CFitnessList
    Dim fitnessSum As Long
    Dim currentValue As Long
    Dim i As Long

    ' Get a list of "fit" tiles
    Set list = FindFitIntersection(topTile, leftTile, rightTile, bottomTile)

    ' Check if no list could be generated
    If list Is Nothing Then
        ' No list could be generated
        ChooseTile = 0
        Exit Function
    End If
    
    ' Determine the total fitness of this list
    For i = 0 To list.Size - 1
        fitnessSum = fitnessSum + list.Fitness(i)
    Next i

    ' Randomly choose a tile from this list
    currentValue = Rnd * fitnessSum
    ' Determine which tile it was
    For i = 0 To list.Size - 1
        ' Update current value based on fitness of current tile
        currentValue = currentValue - list.Fitness(i)
        ' Check if this was the choosen tile
        If currentValue <= 0 Then
            ' This tile was choosen
            ChooseTile = list.tileIndex(i)
            Exit Function
        End If
    Next i
    
    ' No tile choosen. Return 0
    ChooseTile = 0
End Function

' Find a list of tiles that "fits" the given surrounding tiles
' Note: Empty lists are ignored, as are tile indexes of 0
Public Function FindFitIntersection(topTile As Long, leftTile As Long, rightTile As Long, bottomTile As Long) As CFitnessList
    ' Existing lists to merge
    Dim topList As CFitnessList
    Dim leftList As CFitnessList
    Dim rightList As CFitnessList
    Dim bottomList As CFitnessList
    ' New lists generated
    Dim topBottomList As CFitnessList
    Dim leftRightList As CFitnessList
    Dim finalList As CFitnessList
    
    
    ' Obtain existing lists for valid tiles
    ' *************************************

    ' Check if the Top tile is valid
    If topTile <> 0 Then
        ' Get the Bottom list of the Top tile
        Set topList = list(tsBottom, topTile)
    End If

    ' Check if the Left tile is valid
    If leftTile <> 0 Then
        ' Get the Right list of the Left tile
        Set leftList = list(tsRight, leftTile)
    End If

    ' Check if the Right tile is valid
    If rightTile <> 0 Then
        ' Get the Left list of the Right tile
        Set rightList = list(tsLeft, rightTile)
    End If

    ' Check if the Bottom tile is valid
    If bottomTile <> 0 Then
        ' Get the Top list of the Bottom tile
        Set bottomList = list(tsTop, bottomTile)
    End If
    
    
    ' Merge existant lists
    ' ********************
    
    ' Top and bottom
    If topList Is Nothing Then
        Set topBottomList = bottomList
    ElseIf bottomList Is Nothing Then
        Set topBottomList = topList
    Else
        Set topBottomList = IntersectLists(topList, bottomList)
    End If

    ' Left and right
    If leftList Is Nothing Then
        Set leftRightList = rightList
    ElseIf rightList Is Nothing Then
        Set leftRightList = leftList
    Else
        Set leftRightList = IntersectLists(leftList, rightList)
    End If
    
    ' Final results
    If topBottomList Is Nothing Then
        Set finalList = leftRightList
    ElseIf leftRightList Is Nothing Then
        Set finalList = topBottomList
    Else
        Set finalList = IntersectLists(topBottomList, leftRightList)
    End If

    ' Return the final intersected list
    Set FindFitIntersection = finalList
End Function

' Returns the intersection of two fitness lists
Public Function IntersectLists(list1 As CFitnessList, list2 As CFitnessList) As CFitnessList
    Dim newList As New CFitnessList
    Dim listSize As Long
    Dim tile As Long
    Dim fit As Long
    Dim i As Long
    
    ' Add all elements from list1 that are also in list 2
    For i = 0 To list1.Size - 1
        ' Get the tile index from the first list
        tile = list1.tileIndex(i)
        ' Lookup this tile in the second list
        fit = list2.FitnessOfTile(tile)
        ' Check if this tile is also in the second list
        If fit <> 0 Then
            ' Add the fitness of this tile in the first and second lists
            fit = fit + list1.Fitness(i)
            ' Add the tile to the new list
            newList.AddItem tile, fit
        End If
    Next i

    ' Return the new list
    Set IntersectLists = newList
End Function


Property Get FitnessList(tileIndex As Long, dir As tsDirection) As CFitnessList
    Set FitnessList = list(dir, tileIndex)
End Property
